@@ -1,15 +1,15 @@
 # Verilog::Parse.pm -- Verilog preprocessing
-# $Id: Parse.pm,v 1.11 2001/09/17 20:30:58 wsnyder Exp $
-# Author: Wilson Snyder <wsnyder@wsnyder.org>
+# $Id: Parse.pm,v 1.4 1999/06/02 17:30:11 wsnyder Exp $
+# Author: Wilson Snyder <wsnyder@ultranet.com>
 ######################################################################
 #
-# This program is Copyright 2000 by Wilson Snyder.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of either the GNU General Public License or the
-# Perl Artistic License, with the exception that it cannot be placed
-# on a CD-ROM or similar media for commercial distribution without the
-# prior approval of the author.
+# This package implements Verilog Parsing;
+# 
+# This program is Copyright 1998 by Wilson Snyder.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,7 +51,7 @@ and callbacks are called for various entities in the file, as they occur.
 
 This module is no longer supported.  Use Verilog::Parser instead.
 
-=over 4
+=over 
 
 =item $parser->new
 
@@ -130,11 +130,11 @@ that aren\'t covered.
 =head1 DISTRIBUTION
 
 The latest version is available from
-C<http://veripool.com/verilog-perl>.
+C<http://www.ultranet.com/~wsnyder/verilog-perl>.
 
 =head1 AUTHORS
 
-Wilson Snyder <wsnyder@wsnyder.org>
+Wilson Snyder <wsnyder@ultranet.com>
 
 =cut
 
@@ -158,7 +158,7 @@ use Verilog::Language;
 # Other configurable settings.
 $Verilog::Parse::debug = 0;		# for debugging
 
-$VERSION = '1.14';
+( $VERSION ) = '$Revision: 1.4 $ ' =~ /\$Revision:\s+([^\s]+)/;
 
 #######################################################################
 
