@@ -1,15 +1,15 @@
 # Verilog::Parser.pm -- Verilog parsing
-# $Id: Parser.pm,v 1.20 2001/09/17 20:30:58 wsnyder Exp $
-# Author: Wilson Snyder <wsnyder@wsnyder.org>
+# $Id: Parser.pm,v 1.3 1999/06/02 17:30:12 wsnyder Exp $
+# Author: Wilson Snyder <wsnyder@ultranet.com>
 ######################################################################
 #
-# This program is Copyright 2000 by Wilson Snyder.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of either the GNU General Public License or the
-# Perl Artistic License, with the exception that it cannot be placed
-# on a CD-ROM or similar media for commercial distribution without the
-# prior approval of the author.
+# This package implements Verilog Parsing;
+# 
+# This program is Copyright 1998 by Wilson Snyder.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,10 +71,6 @@ preprocessors, pretty-printers, and such.
 
 Return (if $set is undefined) or set current line number.
 
-=item $parser->filename ($set)
-
-Return (if $set is undefined) or set current filename.
-
 =back
 
 In order to make the parser do anything interesting, you must make a
@@ -179,11 +175,11 @@ boundary.  It should be changed to allow arbitrary chunks.
 =head1 DISTRIBUTION
 
 The latest version is available from
-C<http://veripool.com/verilog-perl>.
+C<http://www.ultranet.com/~wsnyder/verilog-perl>.
 
 =head1 AUTHORS
 
-Wilson Snyder <wsnyder@wsnyder.org>
+Wilson Snyder <wsnyder@ultranet.com>
 
 =cut
 
@@ -196,6 +192,7 @@ require Exporter;
 
 use strict;
 use vars qw($VERSION $Debug);
+use English;
 use Carp;
 use FileHandle;
 use Verilog::Language;
@@ -206,7 +203,7 @@ use Verilog::Language;
 # Other configurable settings.
 $Debug = 0;		# for debugging
 
-$VERSION = '1.14';
+$VERSION = sprintf("%d.%03d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
 
 #######################################################################
 
@@ -218,8 +215,7 @@ sub new {
     print "$class->new()\n" if $Debug;
 
     my $self = {unreadback => "",	# Text since last callback
-		line => 1,
-		filename => "UNKNOWN",
+		line => 0,
 		incomment => 0,
 		inquote => 0,
 		preprocess => 0,
@@ -249,7 +245,7 @@ sub unreadback {
 }
 
 sub line {
-    # Return or set line number
+    # Return any un read text and clear it
     my $self = shift;	# Parser
     if (@_) {
 	$self->{line} = shift;
@@ -257,15 +253,6 @@ sub line {
     return $self->{line};
 }
 
-sub filename {
-    # Return or set filename
-    my $self = shift;	# Parser
-    if (@_) {
-	$self->{filename} = shift;
-    }
-    return $self->{filename};
-}
-
 #######################################################################
 
 sub comment {
@@ -314,104 +301,104 @@ sub number {
 
 sub parse {
     # Parse a string
-    # This currently presumes the input is 1 line or longer, but isn't called on a sub-line
     @_ == 2 or croak 'usage: $parser->parse($string)';
     my $self = shift;
     my $text = shift;
 
-    # All state must be changed before the callback.  This allows the callback
-    # to call the parser recursively (such as for `include files).
-
-    while ($text ne "") {
-	print "Lnc $text" if ($Debug);
-	if ($text =~ s/^(\s*\n)//) {
-	    $self->{line}++;
-	    if ($self->{incomment}
-		|| $self->{inquote}) {
-		$self->{token_string} .= $1;
-	    } else {
-		$self->{unreadback} .= $1;
-	    }
-	}
-	elsif ($self->{incomment}) {
-	    if ($text =~ s/^( [^\n]*? \*\/ )//x) {
-		$self->{token_string} .= $1;
-		$self->{incomment} = 0;
-		my $token = $self->{token_string};
-		print "GotaCOMMENT $token\n"    if ($Debug);
-		$self->comment ($token);
-	    }
-	    elsif ($text =~ s/^([^\n]*)// ) {
-		$self->{token_string} .= $1;
-	    }
-	}
-	elsif ($self->{inquote}) {
-	    # Check for strings
-	    if ($text =~ s/^([^\n]*?\")//) {
-		$self->{token_string} .= $1;
-		if ($self->{token_string} !~ /\\\"$/) {	# \"
-		    $self->{inquote} = 0;
+    my $line;
+    foreach $line (split /\n/, $text) {
+	# Keep parsing whatever is on this line
+	$self->{line} ++;
+	while ($line) {
+	    print "Lnc $line\n" if ($Debug);
+	    if ($self->{incomment}) {
+		if ($line =~ /\*\//) {
+		    $self->{token_string} .= $PREMATCH . $MATCH;
+		    $line = $POSTMATCH;
 		    my $token = $self->{token_string};
-		    print "GotaSTRING $token\n"    if ($Debug);
-		    $self->string ($token);
+		    print "GotaCOMMENT $token\n"    if ($Debug);
+		    $self->comment ($token);
+		    $self->{incomment} = 0;
+		}
+		else {
+		    $self->{token_string} .= $line;
+		    $line = "";
 		}
-	    } elsif ($text =~ s/^([^\n]*)//) {
-		$self->{token_string} .= $1;
 	    }
-	}
-	else {
-	    # not in comment
-	    # Strip leading whitespace
-	    if ($text =~ s/^(\s+)//) {
-		$self->{unreadback} .= $1;
+	    elsif ($self->{inquote}) {
+		# Check for strings
+		if ($line =~ /\"/) {
+		    $self->{token_string} .= $PREMATCH . $MATCH;
+		    $line = $POSTMATCH;
+		    if ($PREMATCH !~ /\\$/) {
+			my $token = $self->{token_string};
+			print "GotaSTRING $token\n"    if ($Debug);
+			$self->string ($token);
+			$self->{inquote} = 0;
+		    }
+		} else {
+		    $self->{token_string} .= $line;
+		}
 	    }
-	    if ($text =~ /^\n/) {
-		#Fall though
-	    }
-	    elsif ($text =~ s/^\"//) {
-		$self->{token_string} = "\"";
-		$self->{inquote} = 1;
-	    }
-	    elsif (($text =~ s/^([a-zA-Z_\`\$][a-zA-Z0-9_\`\$]*)//)
-		   || ($text =~ s/^(\\\S+\s+)//)) { #cseddon - escaped identifiers
-		my $token = $1;
-		if (!$self->{inquote}) {
-		    if (Verilog::Language::is_keyword($token)) {
-			print "GotaKEYWORD $token\n"    if ($Debug);
-			$self->keyword ($token);
+	    else {
+		# not in comment
+		# Strip leading whitespace
+		if ($line =~ s/^(\s+)//) {
+		    $self->{unreadback} .= $MATCH;
+		}
+		next if ($line eq "");
+		if ($line =~ /^\"/) {
+		    $line = $POSTMATCH;
+		    $self->{token_string} = $MATCH;
+		    $self->{inquote} = 1;
+		}
+		elsif (($line =~ /^[a-zA-Z_\`\$][a-zA-Z0-9_\`\$]*/)
+                       || ($line =~ /^\\\S+\s+/)) { #cseddon - escaped identifiers
+		    my $token = $MATCH;
+		    $line = $POSTMATCH;
+		    if (!$self->{inquote}) {
+			if (Verilog::Language::is_keyword($token)) {
+			    print "GotaKEYWORD $token\n"    if ($Debug);
+			    $self->keyword ($token);
 			} else {
 			    print "GotaSYMBOL $token\n"    if ($Debug);
 			    $self->symbol ($token);
 			}
 		    }
-	    }
-	    elsif ($text =~ s/^\/\*//) {
-		$self->{token_string} = "\/\*";
-		$self->{incomment} = 1;
-	    }
-	    elsif ($text =~ s/^(\/\/[^\n]*)//) {
-		my $token = $1;
-		print "GotaCOMMENT $token\n"    if ($Debug);
-		$self->comment ($token);
-	    }
-	    elsif (($text =~ s/^(&& | \|\| | == | != | <= | >=
-				 | << | >> | \+: | \-: | \*\* )//x)
-		   || ($text =~ s/^( [][:;@\(\),.%!=<>?|&{}~^+---\/*\#] )//x)) {  #]
-		my $token = $1;
-		print "GotaOPERATOR $token\n"    if ($Debug);
-		$self->operator ($token);
-	    }
-	    elsif (($text =~ s/^([0-9]*'[bhod]\ *[0-9A-FXZa-fxz_?]+)//)    #'
-				 || ($text =~ s/^([0-9]+[0-9a-fA-F_]*)// )) {
-		my $token = $1;
-		print "GotaNUMBER $token\n"    if ($Debug);
-		$self->number ($token);
-	    }
-	    elsif ($text =~ s/^([^\n]+)//) {
-		my $token = $1;
-		$self->{unreadback} .= $token;
-		carp $self->{line} . ":Unknown symbol, ignoring to eol: $token\n";
-	    }
+		}
+		elsif ($line =~ /^\/\*/) {
+		    $self->{token_string} = $MATCH;
+		    $line = $POSTMATCH;
+		    $self->{incomment} = 1;
+		}
+		elsif ($line =~ /^\/\//) {
+		    my $token = $line;
+		    print "GotaCOMMENT $token\n"    if ($Debug);
+		    $self->comment ($token);
+		    $line = "";
+		}
+		elsif (($line =~ /^(&& | \|\| | == | != | <= | >= | << | >> )/x)
+		       || ($line =~ /^([][:;@\(\),.%!=<>?|&{}~^+---\/*\#])/)) {  #]
+		    my $token = $MATCH;
+		    $line = $POSTMATCH;
+		    print "GotaOPERATOR $token\n"    if ($Debug);
+		    $self->operator ($token);
+		}
+		elsif (($line =~ /^([0-9]*'[bhod]\ *[0-9A-FXZa-fxz_?]+)/)    #'
+		    || ($line =~ /^([0-9]+[0-9a-fA-F_]*)/ )) {
+		    my $token = $MATCH;
+		    $line = $POSTMATCH;
+		    print "GotaNUMBER $token\n"    if ($Debug);
+		    $self->number ($token);
+		}
+		else {
+		    if ($line ne "") {
+			$self->{unreadback} .= $line;
+		        carp $self->{line} . ":Unknown symbol, ignoring to eol: $line\n";
+	                $line = "";
+                    }
+		}
+            }
         }
     }
     return $self;
@@ -427,8 +414,6 @@ sub parse_file {
 
     my $fh = new FileHandle;
     $fh->open($filename) or croak "%Error: $! $filename";
-    $self->filename($filename);
-    $self->line(1);
     my $line;
     while ($line = $fh->getline() ) {
 	$self->parse ($line);
