@@ -1,15 +1,15 @@
 # Verilog::SigParser.pm -- Verilog signal parsing
-# $Id: SigParser.pm,v 1.12 2001/09/17 20:30:58 wsnyder Exp $
-# Author: Wilson Snyder <wsnyder@wsnyder.org>
+# $Id: SigParser.pm,v 1.2 1999/06/02 17:30:14 wsnyder Exp $
+# Author: Wilson Snyder <wsnyder@ultranet.com>
 ######################################################################
 #
-# This program is Copyright 2000 by Wilson Snyder.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of either the GNU General Public License or the
-# Perl Artistic License, with the exception that it cannot be placed
-# on a CD-ROM or similar media for commercial distribution without the
-# prior approval of the author.
+# This package implements Verilog Signal Parsing;
+# 
+# This program is Copyright 1998 by Wilson Snyder.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -66,8 +66,8 @@ This method is called when a function is defined.
 
 This method is called when a signal is declared.  The first argument,
 $keyword is ('input', 'output', etc), the second argument is the name of
-the signal.  The third argument is the vector bits or "".  The fourth
-argument is the memory bits or "".
+the signal.  The third argument is the vector bits or undef.  The fourth
+argument is the memory bits or undef.
 
 =item $self->instant ( $module, $cell )
 
@@ -75,8 +75,6 @@ This method is called when a instantantiation is defined.  The first
 parameter is the name of the module being instantiated, and the second
 parameter is the name of the cell.
 
-=back
-
 =head1 SEE ALSO
 
 C<Verilog::Parser>, 
@@ -91,11 +89,11 @@ that aren\'t covered.
 =head1 DISTRIBUTION
 
 The latest version is available from
-C<http://veripool.com/verilog-perl>.
+C<http://www.ultranet.com/~wsnyder/verilog-perl>.
 
 =head1 AUTHORS
 
-Wilson Snyder <wsnyder@wsnyder.org>
+Wilson Snyder <wsnyder@ultranet.com>
 
 =cut
 
@@ -119,7 +117,7 @@ use Verilog::Parser;
 # Other configurable settings.
 $Debug = 0;		# for debugging
 
-$VERSION = '1.14';
+$VERSION = sprintf("%d.%03d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 #######################################################################
 
@@ -134,16 +132,12 @@ sub new {
     $self->{last_module}  = undef;
     $self->{last_function} = undef;
     $self->{last_task}    = undef;
+    @{$self->{last_numbers}} = ();
     @{$self->{last_symbols}} = ();
-    $self->{last_vectors} = "";
-    $self->{last_param} = "";
     $self->{is_inst_ok}   = 1;
     $self->{is_pin_ok}    = 0;
     $self->{is_signal_ok} = 1;
     $self->{in_preproc_line} = -1;
-    $self->{in_vector} = 0;
-    $self->{in_param_assign} = 0;
-    $self->{possibly_in_param_assign} = 0;
     $self->{pin_name}    = undef;
 
     bless $self, $class; 
@@ -200,12 +194,6 @@ sub ppdefine {
     my $definition = shift;
 }
 
-sub ppinclude {
-    my $self = shift;
-    my $defvar = shift;
-    my $definition = shift;
-}
-
 ######################################################################
 # Overrides of Verilog::Parser routines
 
@@ -230,25 +218,18 @@ sub keyword {
     if ($self->{in_preproc_line} != $self->line()) {
 	$self->{last_keyword} = $token;
 	@{$self->{last_symbols}} = ();
-	$self->{last_vectors} = "";
+	@{$self->{last_numbers}} = ();
     }
     if ($token eq "end") {
 	# Prepare for next command
 	$self->{last_keyword} = "";
 	@{$self->{last_symbols}} = ();
-	$self->{last_vectors} = "";
+	@{$self->{last_numbers}} = ();
 	$self->{is_inst_ok} = 1;
 	$self->{is_signal_ok} = 1;
 	$self->{is_pin_ok} = 0;
 	$self->{got_preproc} = 0;
     }
-    elsif ($token eq "endtask") {
-	$self->{last_task} = undef;
-    } elsif ($token eq "endmodule") {
-	$self->{last_module} = undef;
-    } elsif ($token eq "endfunction") {
-	$self->{last_function} = undef;
-    }
 }
 
 sub symbol {
@@ -257,21 +238,15 @@ sub symbol {
     my $token = shift;	# What token was parsed
 
     if ($self->{in_preproc_line} != $self->line()) {
-	if ($self->{in_vector} == 1) {
-	    $self->{last_vectors} = $self->{last_vectors} . $token;
-	} elsif ($self->{in_param_assign} == 1) {
-	    $self->{last_param} = $self->{last_param} . $token;
-	} else {
-	    push @{$self->{last_symbols}}, $token;
-	}
+	push @{$self->{last_symbols}}, $token;
     } else {
 	push @{$self->{last_ppitem}}, $token;
     }
     if ($self->{is_pin_ok}) {
 	if ($self->{last_operator} eq ".") {
 	    $self->{pin_name} = $token;
+	    @{$self->{last_numbers}} = ();
 	    @{$self->{last_symbols}} = ();
-	    $self->{last_vectors} = "";
 	}
     }
 }
@@ -282,7 +257,7 @@ sub number {
     my $token = shift;	# What token was parsed
 
     if ($self->{in_preproc_line} != $self->line()) {
-	$self->{last_vectors} = $self->{last_vectors} . $token;
+	push @{$self->{last_numbers}}, $token;
     } else {
 	push @{$self->{last_ppitem}}, $token;
     }
@@ -298,20 +273,7 @@ sub operator {
     #print "Op $token\n" if $Debug;
 
     if ($self->{in_preproc_line} != $self->line) {
-	if ($token eq "]") {
-	    $self->{in_vector} = 0;
-	    $self->{last_vectors} = $self->{last_vectors} . $token;
-	}
-	elsif ($self->{in_vector} == 1) {
-	    $self->{last_vectors} = $self->{last_vectors} . $token;
-	}
-	elsif ($self->{in_param_assign} == 1) {
-	    if ($token eq ")") {
-		$self->{in_param_assign} = 0;
-	    }
-	    $self->{last_param} = $self->{last_param} . $token;
-	}
-	elsif ($token eq "("
+	if ($token eq "("
 	    && ($lkw eq "" || $lkw =~ /^end/ || $self->{got_preproc})
 	    && (defined $self->{last_symbols}[0])
 	    && (defined $self->{last_symbols}[1])
@@ -328,7 +290,10 @@ sub operator {
 	    if ($self->{is_pin_ok}
 		&& defined $self->{last_symbols}[0]) {
 		my $vec = "";
-		$vec = $self->{last_vectors} if ($self->{last_vectors} ne "");
+		if ($#{$self->{last_numbers}} >= 1) {
+		    $vec = "[" . $self->{last_numbers}[0]
+			. ":" . $self->{last_numbers}[1] . "]";
+		}
 		$self->{is_pin_ok}++;
 		my $pin_name = $self->{pin_name};
 		$pin_name ||= "pin" . $self->{is_pin_ok};
@@ -354,6 +319,13 @@ sub operator {
 		    print "Gota$lkw $mod\n"    if ($Debug);
 		    $self->function ($lkw, $mod);
 		}
+		elsif ($lkw eq "endtask") {
+		    $self->{last_task} = undef;
+		} elsif ($lkw eq "endmodule") {
+		    $self->{last_module} = undef;
+		} elsif ($lkw eq "endfunction") {
+		    $self->{last_function} = undef;
+		}
 		elsif ((($lkw eq "input")
 			|| ($lkw eq "output")
 			|| ($lkw eq "inout")
@@ -365,12 +337,12 @@ sub operator {
 		    foreach $sig (@{$self->{last_symbols}}) {
 			my $vec = "";
 			my $mem = "";
-			if ($self->{last_vectors} ne "") {
-			    if ($self->{last_vectors} =~ /^(\S+) (\S+)$/) {
-				$vec = $1;
-				$mem = $2;
-			    } else {
-				$vec = $self->{last_vectors};
+			if ($#{$self->{last_numbers}} >= 1) {
+			    $vec = "[" . $self->{last_numbers}[0]
+				. ":" . $self->{last_numbers}[1] . "]";
+			    if ($#{$self->{last_numbers}} >= 3) {
+				$mem = "[" . $self->{last_numbers}[2]
+				    . ":" . $self->{last_numbers}[3] . "]";
 			    }
 			}
 			#print "Gota$lkw $sig $vec $mem\n"    if ($Debug);
@@ -380,7 +352,7 @@ sub operator {
 		# Prepare for next command
 		$self->{last_keyword} = "";
 		@{$self->{last_symbols}} = ();
-		$self->{last_vectors} = "";
+		@{$self->{last_numbers}} = ();
 		$self->{is_inst_ok} = 1;
 		$self->{is_signal_ok} = 1;
 		$self->{is_pin_ok} = 0;
@@ -391,23 +363,6 @@ sub operator {
 	    $self->{is_signal_ok} = 0;
 	    $self->{is_inst_ok} = 0;
 	}
-	elsif ($token eq "[") {
-	    $self->{in_vector} = 1;
-	    if ($self->{last_vectors} eq "") {
-		$self->{last_vectors} = $token;
-	    } else {
-		$self->{last_vectors} = $self->{last_vectors} . ' ' . $token;
-	    }
-	}
-	elsif ($token eq "#") {
-	    $self->{possibly_in_param_assign} = 1;
-	    $self->{last_param} = $token;
-	}
-	elsif ($token eq "(" && $self->{possibly_in_param_assign}) {
-	    $self->{in_param_assign} = 1;
-	    $self->{possibly_in_param_assign} = 0;
-	    $self->{last_param} = $self->{last_param} . $token;
-	}
 	else {
 	    $self->{is_inst_ok} = 0;
 	}
